#!/usr/bin/env python

"""
Moin wiki serialisers.

Copyright (C) 2017 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from cgi import escape

class Serialiser:

    "General serialisation support."

    def __init__(self, out):
        self.out = out

class MoinSerialiser(Serialiser):

    "Serialisation of the page."

    def start_region(self, level, indent, type):
        out = self.out
        if level:
            out(" " * indent + "{" * level)
        if type and level:
            out("#!%s\n" % type)

    def end_region(self, level, indent, type):
        out = self.out
        if level:
            out("}" * level)

    def start_block(self, final):
        pass

    def end_block(self, final):
        if not final:
            self.out("\n")

    def start_listitem(self):
        self.out(" *")

    def end_listitem(self):
        pass

    def rule(self, length):
        self.out("-" * length)

    def text(self, s):
        self.out(s)

class HTMLSerialiser(Serialiser):

    "Serialisation of the page."

    def start_region(self, level, indent, type):
        l = []
        out = l.append
        if level:
            out("level-%d" % level)

        if indent:
            out("indent-%d" % indent)

        # NOTE: Encode type details for CSS.

        if type:
            out("type-%s" % escape(type, True))

        self.out("<span class='%s'>" % " ".join(l))

    def end_region(self, level, indent, type):
        self.out("</span>")

    def start_block(self, final):
        self.out("<p>")

    def end_block(self, final):
        self.out("</p>")

    def start_listitem(self):
        self.out("<li>")

    def end_listitem(self):
        self.out("</li>")

    def rule(self, length):
        self.out("<hr style='height: %dpt' />" % min(length, 10))

    def text(self, s):
        self.out(escape(s))

# Top-level functions.

def serialise(doc, serialiser=MoinSerialiser):
    l = []
    doc.to_string(serialiser(l.append))
    return "".join(l)

# vim: tabstop=4 expandtab shiftwidth=4
